//=========================================================================
//
//	FILE:				CP_DialogModality.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to work in conjunction with the CP_Dialog classes
//						to handle the event handling and modality of the dialog.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 198 $
//						Latest Checkout:  $Date: 2005-06-01 23:15:21 -0700 (Wed, 01 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_DialogModality
#define _H_CP_DialogModality
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Window.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DialogModality ===
#pragma mark -
#endif

/*!
*	@class		CP_DialogModality
*	@brief		Base class for setting and handling the modality of a CP_Dialog.
*/
class CP_EXPORT CP_DialogModality
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DialogModality( CP_Window *inWindow );
	//! Destructor.
	virtual								~CP_DialogModality() throw();
	//@}

	//! @name Display
	//@{
										/*!
										*	Display the dialg and begin the event handling loop for
										*	the dialog.
										*
										*	@param		inAutoPosition		Used to control the positioning
										*									of the dialog.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual void						Display( bool /*inAutoPosition = true*/ )	{}
	//@}

	//! @name Dismissal
	//@{
										/*!
										*	Handles dismissal of the dialog.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
	virtual void						Dismiss()									{}
	//@}
	
	//! @name Getters
	//@{
										/*!
										*	Returns the window being used by the dialog.
										*	
										*	@exception	none
										*	
										*	@return		CP_Window *		The dialogs main window.
										*/
			CP_Window 					*GetWindow()								{ return( fWindow ); }
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the window. This should be the same window that is the
										*	CP_Dialogs main window.
										*
										*	@param		inWindow		The window used by the dialog.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void 						SetWindow( CP_Window *inWindow )			{ fWindow = inWindow; }
	//@}

protected:

	// class data
protected:
	CP_Window 							*fWindow;				/*! CP_Window used in displaying the dialog. */

	#if TARGET_OS_WIN32
		CP_E_AppStatus					fDialogStatus;			/*! Current state of the dialog run loop. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Dialogn Status
	//@{
										/*!
										*	Returns the current status of the dialogs event loop.
										*	
										*	@exception	none
										*	
										*	@return		CP_E_AppStatus	Indicates the status of the dialogs
										*								event handling.
										*/
			CP_E_AppStatus				GetDialogStatus()							{ return( fDialogStatus ); }
	//@}

	//! @name Events
	//@{
			void						RunOneEventLoop( CP_Event<kCP_WINDOW_EVENT, 0>& inOutEvent );
			bool						GetAnEvent( CP_Event<kCP_WINDOW_EVENT, 0>& inOutEvent );
			void						DispatchEvent( CP_Event<kCP_WINDOW_EVENT, 0>& inEvent );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DialogModality_AppModless ===
#pragma mark -
#endif

/*!
*	@class		CP_DialogModality_AppModless
*	@brief		Dialog modality class for dialogs that are displayed in an application modless style.
*/
class CP_EXPORT CP_DialogModality_AppModless : public CP_DialogModality
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DialogModality_AppModless( CP_Window *inWindow );
	//! Destructor.
	virtual								~CP_DialogModality_AppModless() throw();
	//@}

	//! @name Display
	//@{
	virtual void						Display( bool inAutoPosition = true );
	//@}

	//! @name Dismissal
	//@{
	virtual void						Dismiss();
	//@}
	
protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DialogModality_AppModal ===
#pragma mark -
#endif

/*!
*	@class		CP_DialogModality_AppModal
*	@brief		Dialog modality class for dialogs that are displayed in an application modal style.
*/
class CP_EXPORT CP_DialogModality_AppModal : public CP_DialogModality
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DialogModality_AppModal( CP_Window *inWindow );
	//! Destructor.
	virtual								~CP_DialogModality_AppModal() throw();
	//@}

	//! @name Display
	//@{
	virtual void						Display( bool inAutoPosition = true );
	//@}

	//! @name Dismissal
	//@{
	virtual void						Dismiss();
	//@}
	
protected:

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CP_DialogModality_WindowModal ===
#pragma mark -
#endif

/*!
*	@class		CP_DialogModality_WindowModal
*	@brief		Dialog modality class for dialogs that are displayed in an window modal style.
*/
class CP_EXPORT CP_DialogModality_WindowModal : public CP_DialogModality
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DialogModality_WindowModal(
												CP_Window *inWindow,
												CP_Window *inParentWindow );
	//! Destructor.
	virtual								~CP_DialogModality_WindowModal() throw();
	//@}

	//! @name Display
	//@{
	virtual void						Display( bool inAutoPosition = true );
	//@}

	//! @name Dismissal
	//@{
	virtual void						Dismiss();
	//@}
	
protected:

	// class data
protected:
	CP_Window 							*fParentWindow;		/*! Parent window for the dialogs window. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DialogModality

