//=========================================================================
//
//	FILE:				CP_LogFile.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to handle logging of messages to either a file or a
//						debugger.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 145 $
//						Latest Checkout:  $Date: 2005-04-14 16:01:09 -0700 (Thu, 14 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_LogFile
#define _H_CP_LogFile
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Types.h"
#include "CP_String.h"
#include "CP_Mutex.h"

	// Standard library
#include <stdio.h>

CPLAT_Begin_Namespace_CPLAT

enum CP_E_LogFile_LogOptions
{
	CP_E_LogOptions_ExplicitPath = 1,
	CP_E_LogOptions_TempDirectory = 2, 
	CP_E_LogOptions_LogDirectory = 4, 
	CP_E_LogOptions_SystemLogDirectory = 8, 
	CP_E_LogOptions_DeleteIfEmpty = 16,
	CP_E_LogOptions_AppendFile = 32,
	CP_E_LogOptions_LogToDebug = 64,
	CP_E_LogOptions_LogToConsole = 128,
	CP_E_LogOptions_LogToSyslog = 256
};

enum CP_E_LogFile_Options
{
	CP_E_LogFile_ShowNone = 0,
	CP_E_LogFile_ShowTimeStamp = 2,
	CP_E_LogFile_ShowLogLevel = 4, 
	CP_E_LogFile_ShowCount = 8,
	CP_E_LogFile_ShowAll = 15, 
	CP_E_LogFile_IndentByLevel = 256
};

enum CP_E_LogFile_Status
{
	CP_E_LogStatus_NotCP_E_LogStatus_Ready,
	CP_E_LogStatus_Ready,
	CP_E_LogStatus_Paused
};

enum CP_E_LogFile_ReturnValues
{
	CP_E_LogReturn_Success,
	CP_E_LogReturn_UnableToOpenFile,
	CP_E_LogReturn_NotLogging,
	CP_E_LogReturn_NotPaused,
	CP_E_LogReturn_AlreadyStarted, 
	CP_E_LogReturn_NoLoggingModeSpecified,
	CP_E_LogReturn_TooManyModesSpecified
};

//!  CP_LogFile
/*!
  Class to handle logging of messages to either a file or a debugger. 
*/
class CP_EXPORT CP_LogFile
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_LogFile();
	//! Destructor.
	virtual								~CP_LogFile();
	//@}
											
	//! @name Control
	//@{
			SInt32						Start(
												const char *inFile, 
												UInt32 inType = CP_E_LogOptions_ExplicitPath | CP_E_LogOptions_LogToDebug,
												UInt32 inOptions = CP_E_LogFile_ShowAll,
												UInt32 inLevel = 0 );
			SInt32						Start(
												const CP_FileSystemSpec& inFileSpec, 
												UInt32 inType = CP_E_LogOptions_ExplicitPath | CP_E_LogOptions_LogToDebug,
												UInt32 inOptions = CP_E_LogFile_ShowAll,
												UInt32 inLevel = 0 );
			SInt32						Start(
												UInt32 inType = CP_E_LogOptions_LogToDebug,
												UInt32 inOptions = CP_E_LogFile_ShowAll,
												UInt32 inLevel = 0 );
			SInt32						Pause();
			SInt32						Continue();
			SInt32						Stop();
	//@}
			
	//! @name Logging
	//@{
										/*!
										*	Log a message to the log.
										*	
										*	@param		inMsg 		The text of the message.
										*	@param		inLevel		The indent level of the message.
										*
										*	@exception	none
										*	
										*	@return		SInt32			Returns one of the status messages to indicate
										*								the sucess of writing the message. Currently
										*								we only return CP_E_LogReturn_Success.
										*/
			SInt32						LogMessage(
												const char *inMessage,
												   UInt32 inLevel = 0 )					{ return LogMessageSelf(inMessage, inLevel); }
			SInt32						LogMessage(
												UInt32 inLevel,
												const char *inFormatStr, ... );
			SInt32						VLogMessage(
												UInt32 inLevel,
												const char *inFormatStr,
												va_list args );
	//@}

										/*!
										*	Set the level of the error, which controls the level of indenting in the file.
										*	
										*	@param		inLevel			The level of indenting to use.
										*
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetLevel( UInt32 inLevel )						{ fLevel = inLevel; }
										/*! Return the level. */

										/*!
										*	Returns the level of the error.
										*
										*	@exception	none
										*	
										*	@return		UInt32			The level of indenting.
										*/
			UInt32						GetLevel() const								{ return( fLevel ); }

										/*!
										*	Returns the logging mode in use, which controls where / how the log file is created.
										*
										*	@exception	none
										*	
										*	@return		UInt32			The mode in use.
										*/
			UInt32						GetMode() const									{ return( fMode ); }
			
										/*!
										*	Sets the options that control varius aspects of how the logging
										*	takes place. The following are valid options to pass in:
										*		- CP_E_LogOptions_ExplicitPath
										*		- CP_E_LogOptions_TempDirectory
										*		- CP_E_LogOptions_LogDirectory
										*		- CP_E_LogOptions_SystemLogDirectory 
										*		- CP_E_LogOptions_DeleteIfEmpty
										*		- CP_E_LogOptions_AppendFile
										*		- CP_E_LogOptions_LogToDebug
										*		- CP_E_LogOptions_LogToConsole
										*		- CP_E_LogOptions_LogToSyslog
										*
										*
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetOptions( UInt32 inOptions )					{ fOptions = inOptions; }

										/*!
										*	Returns the logging options in effect.
										*
										*	@exception	none
										*	
										*	@return		UInt32		The current logging options.
										*/
			UInt32						GetOptions() const								{ return fOptions; }

										/*!
										*	Returns the current status of the log file, and can be one of the
										*	following values:
										*		- CP_E_LogStatus_NotCP_E_LogStatus_Ready,
										*		- CP_E_LogStatus_Ready,
										*		- CP_E_LogStatus_Paused
										*
										*	@exception	none
										*	
										*	@return		CP_E_LogFile_Status		The current log file status.
										*/
			CP_E_LogFile_Status			GetStatus()	const								{ return( fStatus ); }
			
										/*!
										*	Returns the file name of the log.
										*
										*	@exception	none
										*	
										*	@return		const char *	The file name of the log.
										*/
			const char					*GetFilename() const							{ return( fFile ); }

			void						SetSyslogPriorityThreshold(
												int inSysLogPriority,
												UInt32 inLogLevelThreshold );
			void						SetSyslogFacility( int inSysLogFacility );

protected:
	//! @name Logging
	//@{
	virtual SInt32						LogMessageSelf(
												const char *inMessage,
												UInt32 inLevel );
	//@}

private:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
											// Don't allow copy and assignment
										CP_LogFile( const CP_LogFile& );
	//@}

	//! @name operators
	//@{
	CP_LogFile&							operator = ( const CP_LogFile& );
	//@}

	// class data
protected:
	FILE								*fOutfile;
	UInt32								fOptions;
	UInt32								fLevel;
	CP_E_LogFile_Status					fStatus;
	UInt32								fMode;
	UInt32								fCount;
	char								fFile[ 255 ];
	bool								fEmptyFileFlag;
	bool								fLogToFileFlag;
	char								fMessage[ 1024 ];
	char								fBuffer[ 1024 ];
	char								fBuffer1[ 1024 ];
	UInt32								fSyslogPriorityThresholds[32];
	int									fSyslogFacility;
	
	CP_Mutex							fBufferMutex;				//!< mutex to synchronize access to fBuffer.
	CP_Mutex							fLogMutex;					//!< mutex to serialize LogMessage calls

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Logging
	//@{
			void						LogMessageToDebugger( char *inMsg ); 
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Logging
	//@{
			void						LogMessageToDebugger( char *inMsg ); 
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Logging
	//@{
			void						LogMessageToDebugger( char *inMsg ); 
	//@}
#endif
};

//!  CP_BufferedLogFile
/*!
  Class to handle buffered 9stream) logging of messages to either a file or a debugger. 
  The messages are buffered until the line ending character is found.
*/

class CP_EXPORT CP_BufferedLogFile : public CP_LogFile
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_BufferedLogFile( char newLineChar );
	//! Destructor.
	virtual								~CP_BufferedLogFile();
	//@}
	
	//! @name Logging
	//@{
										/*!
										*	Flush (write) the buffered message to the log file.
										*
										*	@exception	none
										*	
										*	@return		SInt32		The status of the flushing.
										*/
			SInt32						Flush()								{ return( FlushSelf(false) ); }
	//@}

protected:
	//! @name Logging
	//@{
	virtual SInt32						LogMessageSelf(
												const char *inMessage,
												UInt32 inLevel = 0 );


			SInt32						FlushSelf( bool inMutexLocked );										
	//@}

	// class data
protected:
	CP_String							fStreamBuffer;
	UInt32								fStreamLevel;
	char								fNewLineChar;
	CP_Mutex							fStreamBufferMutex;			//!< mutex to synchronize access to fStreamBuffer
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_LogFile

