//=========================================================================
//
//	FILE:				CP_Strings.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for dealing with strings and string resources
//						used in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 104 $
//						Latest Checkout:  $Date: 2005-03-21 15:01:42 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_Strings
#define _H_CP_Strings
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\namespace	CP_Strings
*	@brief		A group of functions to load strings from resources.
*/
namespace CP_Strings
{
	CP_String							GetString(
												SInt16 inResID,
												SInt16 inIndex );
	
	//! @name Get a localized string from the common table.
	//! @{
	/*!
	 *	Pseudo API to get a localized string from the common string table.
	 *	Fetches the string specified by \a inKey from the common string table (e.g. Localizable.strings on Mac OS).
	 *	The comment parameter is only used by the genstrings tool to construct the string tables. It's hidden from
	 *	the compiler using the macros defined below, which map this API to the comment-less private APIs that do the
	 *	string fetching business.
	 *
	 *	
	 *	@param		inKey			Specifies which string to fetch.
	 *	@param		inComment		Comment that will only appear in the string table to help localizers.
	 *	
	 *	@exception	none
	 *
	 *	@return		CP_String		The localized string or the key string if no localized string was found.
	 */
	CP_String GetLocalizedString(const char* inKey, const char* inComment);
	CP_String GetLocalizedString(const CP_String& inKey, const char* inComment);
	//! @}
	
	//! @name Get a localized string from a specific table.
	//! @{
	/*!
	 *	Pseudo API to get a localized string from the speicified string table.
	 *	Fetches the string specified by \a inKey from the specified string table (e.g. <name>.strings on Mac OS).
	 *	The comment parameter is only used by the genstrings tool to construct the string tables. It's hidden from
	 *	the compiler using the macros defined below, which map this API to the comment-less private APIs that do the
	 *	string fetching business.
	 *	
	 *	
	 *	@param		inKey			Specifies which string to fetch.
	 *	@param		inTable			Specifies which string table to use (i.e. the name of the string table file).
	 *	@param		inComment		Comment that will only appear in the string table to help localizers.
	 *	
	 *	@exception	none
	 *
	 *	@return		CP_String		The localized string or the key string if no localized string was found.
	 */
	CP_String GetLocalizedStringFromTable(const char* inKey, const char* inTable, const char* inComment);
	CP_String GetLocalizedStringFromTable(const CP_String& inKey, const char* inTable, const char* inComment);
	//! @}
	
	//! Macro to map GetLocalizedString call with comment string to 'real' API, hiding the comment
	#define GetLocalizedString(str, comment) _CP_GetLocalizedString(str)
	//! Macro to map GetLocalizedStringFromTable call with comment string to 'real' API, hiding the comment
	#define GetLocalizedStringFromTable(str, table, comment) _CP_GetLocalizedString_FromTable(str, table)

	//========================================================================
	// Platform specific routines
	//========================================================================
	#if TARGET_OS_MAC
	#endif

	#if TARGET_OS_WIN32
	#endif

	#if TARGET_OS_LINUX
	#endif

	/// @if 0
	// private APIs -> use GetLocalizedString macros instead
	CP_String _CP_GetLocalizedString(const CP_String& inString);
	CP_String _CP_GetLocalizedString(const char* inString);
	CP_String _CP_GetLocalizedString_FromTable(const CP_String& inString, const CP_String& inTable);
	CP_String _CP_GetLocalizedString_FromTable(const CP_String& inString, const char* inTable);
	CP_String _CP_GetLocalizedString_FromTable(const char* inString, const char* inTable);
	CP_String _CP_GetLocalizedString_FromTable(const char* inString, const CP_String& inTable);
	/// @endif
	
}	// namespace CP_Strings

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Strings

