//=========================================================================
//
//	FILE:				CP_CFMutableObject.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle a MacOS CFMutableObject.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 271 $
//						Latest Checkout:  $Date: 2005-09-06 19:24:58 -0700 (Tue, 06 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_CFMutableObject
#define _H_CP_CFMutableObject
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_CFObject.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class	CP_CFMutableObject
*	\brief	A class to handle a MacOS CFMutableObject.
*/
template <typename TCFRef, typename TMutableCFRef>
class CP_EXPORT CP_CFMutableObject : public CP_CFObject<TCFRef>
{
private:
	typedef CP_CFObject<TCFRef>	T_Base;

public:
	//! @name Getters
	//@{
	TMutableCFRef						GetMutableCFRef();

										/*!
										*	operator TMutableCFRef to return the mutable CFRef object
										*	for this class.
										*	
										*	@exception	none
										*	
										*	@return		CFMutableCFRef		The mutable CFRef object for ths class.
										*/
								operator TMutableCFRef()								{ return( GetMutableCFRef() ); }
	//@}

	//! @name Reference
	//@{
	void								AttachRefMutable(
												TMutableCFRef inMutableRef,
												bool inRetain );
	void								AttachRef(
								  				TCFRef inRef,
								 				bool inRetain );
						
	TMutableCFRef						DetachRefMutable();
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_CFMutableObject();
										CP_CFMutableObject(
												TCFRef inCFRef,
												bool inRetain );
										CP_CFMutableObject(
												TMutableCFRef inCFRef,
												bool inRetain );
								
										CP_CFMutableObject( const CP_CFMutableObject& inMutableObject );
	//! Destructor.
	virtual								~CP_CFMutableObject() throw();
	//@}

	//! @name Mutable
	//@{
	void 								MakeMutable();
			
	TMutableCFRef						MutableCopy( TCFRef inCFRef );
	//@}
	
	// class data
private:
	bool								fIsMutable;		/* ! Flag to indicate if we are dealing with a mutable object. */
};

//=========================================================================
//	FUNCTION:	CP_CFMutableObject
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
CP_CFMutableObject<TCFRef, TMutableCFRef>::CP_CFMutableObject()
{
	fIsMutable = false;
}

//=========================================================================
//	FUNCTION:	CP_CFMutableObject
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inRef					The CFRef we are storing in this class.
*	@param		inRetain				Flag to indicate if we retain the reference.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
CP_CFMutableObject<TCFRef, TMutableCFRef>::CP_CFMutableObject( TCFRef inRef, bool inRetain )
	: T_Base( inRef, inRetain )
{
	fIsMutable = false;
}

//=========================================================================
//	FUNCTION:	CP_CFMutableObject
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inRef					The CFMutableRef we are storing in this class.
*	@param		inRetain				Flag to indicate if we retain the reference.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
CP_CFMutableObject<TCFRef, TMutableCFRef>::CP_CFMutableObject( TMutableCFRef inRef, bool inRetain )
: T_Base( inRef, inRetain )
{
	fIsMutable = true;
}

//=========================================================================
//	FUNCTION:	CP_CFMutableObject
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inObject				Object to copy the fields from.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
CP_CFMutableObject<TCFRef, TMutableCFRef>::CP_CFMutableObject( const CP_CFMutableObject& inObject )
	: T_Base( inObject )
{
	fIsMutable = inObject.fIsMutable;
}

//=========================================================================
//	FUNCTION:	~CP_CFMutableObject
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
CP_CFMutableObject<TCFRef, TMutableCFRef>::~CP_CFMutableObject() throw()
{
}

//=========================================================================
//	FUNCTION:	MakeMutable
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Turns this object we are storing into a mutable version.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
void CP_CFMutableObject<TCFRef, TMutableCFRef>::MakeMutable()
{
	if( !fIsMutable ) 
	{
		AttachRefMutable(MutableCopy(const_cast<TMutableCFRef>(T_Base::GetCFRef())/*GetCFRef()*/), false);

		fIsMutable = true;
	}
}

//=========================================================================
//	FUNCTION:	GetMutableCFRef
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Returns a mutable CFRef for this object. If the object is not mutable already,
*	then it is first made mutable before returning the CFRef.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
TMutableCFRef CP_CFMutableObject<TCFRef, TMutableCFRef>::GetMutableCFRef()
{
	MakeMutable();
	
	return( const_cast<TMutableCFRef>(T_Base::GetCFRef()) );
}

//=========================================================================
//	FUNCTION:	AttachRef
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Attach a reference to the passed in CFRef.
*
*	@param		refToAttach				The object to attach the reference to.
*	@param		inRetain				Flag to indicate if we retain this object.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
void CP_CFMutableObject<TCFRef, TMutableCFRef>::AttachRef( TCFRef refToAttach, bool inRetain )
{
	T_Base::AttachRef( refToAttach, inRetain );

	fIsMutable = false;
}

//=========================================================================
//	FUNCTION:	AttachRefMutable
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Attach a reference to the passed in CFRef.
*
*	@param		inMutableRef			The mutable ref object to attach the reference to.
*	@param		inRetain				Flag to indicate if we retain this object.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
void CP_CFMutableObject<TCFRef, TMutableCFRef>::AttachRefMutable( TMutableCFRef inMutableRef, bool inRetain )
{
	TMutableCFRef			refToAttach = inMutableRef;

	if( inRetain && (inMutableRef != NULL) )
		refToAttach = MutableCopy( inMutableRef );
	
	T_Base::AttachRef( refToAttach, false );
	
	fIsMutable = (refToAttach != NULL);
}

//=========================================================================
//	FUNCTION:	DetachRefMutable
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Detach a reference to the CFRef maintained by this object. If the object
*	is not mutable, it is first made mutable.
*	
*	@exception	none
*/
template <typename TCFRef, typename TMutableCFRef>
TMutableCFRef CP_CFMutableObject<TCFRef, TMutableCFRef>::DetachRefMutable()
{
	MakeMutable();
	
	return( const_cast<TMutableCFRef>(T_Base::DetachRef() ) );
}

CPLAT_End_Namespace_CPLAT

#endif  // TARGET_OS_MAC

#endif	// _H_CP_CP_CFMutableObject

