//=========================================================================
//
//	FILE:				CP_Splitter.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a splitter view that can contain 2
//						other views, and display them vertically or horizontally.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 203 $
//						Latest Checkout:  $Date: 2005-06-04 13:40:08 -0700 (Sat, 04 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_Splitter
#define _H_CP_Splitter
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
	const ControlPartCode kControlPartSubViewA		= 1;
	const ControlPartCode kControlPartSplitter		= 2;
	const ControlPartCode kControlPartSubViewB		= 3;
#endif

/*!
*	@class		CP_Splitter
*	@brief		Implementation of a slitter view.
*
*				In a splitter view there are 2 views that are managed by the splitter,
*				the left view and the right view. A splitter can have either a vertical
*				or horizontal orientation. If the splitter is arranged so the views are
*				horizontal, then the left view is the top view, and the right view is the
*				bottom view.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Splitter : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Splitter();
										CP_Splitter(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inIsVertical,
												float inSplitRatio );
	//! Destructor.
	virtual								~CP_Splitter() throw();
	//@}

	//! @name Getters
	//@{
			CP_View						*GetSubViewA()										{ return( fSubViewA ); }
			CP_View						*GetSubViewB()										{ return( fSubViewB ); }

			float						GetMinSplitRatio()									{ return( fMinSplitRatio ); }
			float						GetMaxSplitRatio()									{ return( fMaxSplitRatio ); }
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Setters
	//@{
			void						SetSubViewA( CP_View *inSubViewA );
			void						SetSubViewB( CP_View *inSubViewB );

			void						SetMinSplitRatio( float inRatio );
			void						SetMaxSplitRatio( float inRatio );

			void						SetDefaultSplitterWidth( SInt16 inWidth );
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );
	//@}

	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Frame
	//@{
	virtual	void						HandleBoundsChanged(
												const CP_Rect& inPreviousBounds,
												const CP_Rect& inNewBounds );
	//@}

	//! @name Getters
	//@{
	virtual	float						GetVSplitterWidth();
	virtual	float						GetHSplitterHeight();
	//@}

	//! @name Misc
	//@{
	virtual	void						CalculateRects(
												CP_Rect *outRectA,
												CP_Rect *outRectB,
												CP_Rect *outSplitterRect );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
	//! @name Misc
	//@{
			void						AdjustViews();
	//@}

	// class data
protected:
	CP_View								*fSubViewA;				/*! The top or left view. */
	CP_View								*fSubViewB;				/*! The bottom or right view. */
	
	bool								fIsVertical;			/*! If true the views are layout left to right, vs top to bottom. */
	float								fSplitRatio;			/*! Ratio of left to right view. */
	
	float								fMinSplitRatio;			/*! Minimum ratio left view can be set to. */
	float								fMaxSplitRatio;			/*! Maximum ratio left view can be set to. */

	SInt16								fDefaultSplitterWidth;	/*! The default width of the splitter indicator. */
	
	#if TARGET_OS_WIN32
		float							fLastRatio;
		CP_Point						fLastWhere;
		float							fClickOffsetMin;
		float							fClickOffsetMax;
		CP_Rect							fSplitRect;
		bool							fResizing;
	#endif
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Events
	//@{
	virtual	ControlPartCode				HandleHitTest( const CP_Point& inMouseLocation );
	virtual	ControlPartCode				HandleTracking(
												const CP_Point& inMouseLocation,
												ControlActionUPP inActionProc );
	//@}
#endif

#if TARGET_OS_WIN32
public:
			void						HandleMouseTracking(
												const CP_Point& inMouseLocation,
												void *inData );

private:
			void						HandleMouseDown( CP_Event_Base& inEvent );
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_OS_WIN32
		// mouse tracking callbacks
	void CP_Splitter_TrackMouse( CP_View *inDestView, CP_Point& inMouseLocation, void *inData );
#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Splitter
